@@ -38,6 +38,7 @@ from . import (constants,
                separations,
                functors,
 )
+from ._chemical_data import ChemicalData
 from ._chemical import Chemical
 from ._chemicals import Chemicals, CompiledChemicals
 from ._thermal_condition import ThermalCondition
@@ -46,21 +47,23 @@ from .mixture import Mixture
 from ._thermo import Thermo, IdealThermo
 from ._settings import settings
 from ._thermo_data import ThermoData
-from . import (indexer,
-               reaction,
-               equilibrium
+from . import (
+    indexer,
+    reaction,
+    equilibrium
 )
 from ._stream import Stream
 from ._multi_stream import MultiStream
 from .base import functor
 from .reaction import *
+from .equilibrium import * 
 
-__all__ = ('Chemical', 'Chemicals', 'CompiledChemicals', 'Mixture', 'Thermo', 
+__all__ = ('Chemical', 'ChemicalData', 'Chemicals', 'CompiledChemicals', 'Mixture', 'Thermo', 
            'IdealThermo', 'Stream', 'MultiStream', 'ThermalCondition', 
-           'mixture', 'ThermoData', *reaction.__all__, 'indexer', 'settings', 
-           'functor', 'functors', 'chemicals', 'base', 'equilibrium', 
-           'units_of_measure', 'exceptions', 'functional', 'reaction', 
-           'constants', 'utils', 'separations')
+           'mixture', 'ThermoData', *reaction.__all__, *equilibrium.__all__, 
+           'indexer', 'settings', 'functor', 'functors', 'chemicals', 'base', 
+           'equilibrium', 'units_of_measure', 'exceptions', 'functional', 
+           'reaction', 'constants', 'utils', 'separations')
 
 def speed_up():
     from warnings import warn
@@ -81,4 +84,4 @@ pd.set_option('display.max_columns', 10)
 pd.set_option('max_colwidth', 35)
 del np, pd
 
-__version__ = "0.27.14"
+__version__ = "0.27.20"
