@@ -7,6 +7,12 @@
 # for license details.
 """
 """
+try:
+    import CoolProp
+except:    
+    add_ID = True
+else:
+    add_ID = False
 import thermosteam as tmo
 from warnings import warn
 from flexsolve import IQ_interpolation, aitken_secant
@@ -32,6 +38,7 @@ from chemicals.reaction import (
 from chemicals.elements import (
     similarity_variable as compute_similarity_variable, 
     molecular_weight as compute_molecular_weight,
+    charge_from_formula,
     atoms_to_Hill,
     get_atoms,
 )
@@ -57,15 +64,16 @@ from .base import (PhaseHandle, PhaseTHandle, PhaseTPHandle,
 from .units_of_measure import chemical_units_of_measure
 from .utils import copy_maybe, check_valid_ID
 from . import functional as fn 
-from ._phase import check_phase
+from ._phase import check_phase, valid_phases
 from . import units_of_measure as thermo_units
 from chemicals.utils import Z
 from thermo.eos import IG, PR
 from thermo import (
     TDependentProperty, TPDependentProperty,
-    VaporPressure, SublimationPressure,
+    VaporPressure, 
     EnthalpyVaporization,
     SurfaceTension,
+    SublimationPressure, EnthalpySublimation,
     VolumeSolid, VolumeLiquid, VolumeGas, VolumeSupercriticalLiquid,
     HeatCapacitySolid, HeatCapacityLiquid, HeatCapacityGas,
     ThermalConductivityLiquid, ThermalConductivityGas,
@@ -208,7 +216,7 @@ _groups = ('_Dortmund',
            '_PSRK',
            '_NIST')
 
-_model_handles = ('_Psat', '_Hvap', '_sigma', '_epsilon')
+_model_handles = ('_Psat', '_Hvap', '_sigma', '_epsilon', '_Hsub', '_Psub')
     
 _phase_handles = ('_kappa', '_V', '_Cn', '_mu')
 
@@ -253,7 +261,7 @@ class Chemical:
     ----------
     ID : str
         One of the following [-]:
-            * Name, in IUPAC form or common form or a synonym registered in PubChem
+            * Name, in IUPAC form or common form or an alias registered in PubChem
             * InChI name, prefixed by 'InChI=1S/' or 'InChI=1/'
             * InChI key, prefixed by 'InChIKey='
             * PubChem CID, prefixed by 'PubChem='
@@ -336,13 +344,13 @@ class Chemical:
              Tb: 373.12 K
              Tt: 273.15 K
              Tc: 647.14 K
-             Pt: 610.71 Pa
+             Pt: 610 Pa
              Pc: 2.2048e+07 Pa
              Vc: 5.6e-05 m^3/mol
              Hf: -2.8582e+05 J/mol
              S0: 70 J/K/mol
              LHV: -44011 J/mol
-             HHV: 0 J/mol
+             HHV: -0 J/mol
              Hfus: 6010 J/mol
              Sfus: None
              omega: 0.344
@@ -430,6 +438,7 @@ class Chemical:
      'ROWLINSON_BONDI',
      'ROWLINSON_POLING',
      'USER_METHOD',
+     'WEBBOOK_SHOMATE',
      'ZABRANSKY_SPLINE_C']
     >>> Water.Cn.l.method = 'ZABRANSKY_SPLINE_C'
     
@@ -469,7 +478,7 @@ class Chemical:
     """
     __slots__ = ('_ID', '_locked_state', 
                  '_phase_ref', '_eos', 
-                 '_synonyms', *_names, *_groups, 
+                 '_aliases', *_names, *_groups, 
                  *_handles, *_data,
                  '_N_solutes')
     
@@ -499,6 +508,12 @@ class Chemical:
                     data]):
                 warn('cached chemical returned; additional parameters disregarded')
             return chemical_cache[ID]
+        ID = ID.strip()
+        if not phase and len(ID) > 2 and ID[-2] == ',':
+            phase = ID[-1]
+            if phase not in valid_phases:
+                raise ValueError(f'invalid phase {repr(phase)} encountered while parsing ID')
+            ID = ID[:-2]
         search_ID = search_ID or ID
         if not eos: eos = PR
         if search_db:
@@ -549,7 +564,7 @@ class Chemical:
 
     @classmethod
     def blank(cls, ID, CAS=None, phase_ref=None, phase=None,
-              formula=None, synonyms=None, **data):
+              formula=None, aliases=None, synonyms=None, **data):
         """
         Return a new Chemical object without any thermodynamic models or data 
         (unless provided).
@@ -607,18 +622,23 @@ class Chemical:
         
         """
         self = super().__new__(cls)
+        self._ID = ID
         self._eos = None
         self._UNIFAC = UNIFACGroupCounts()
         self._Dortmund = DortmundGroupCounts()
         self._PSRK = PSRKGroupCounts()
         self._NIST = NISTGroupCounts()
-        self.synonyms = synonyms or ()
+        self.aliases = aliases or ()
+        if synonyms: 
+            if isinstance(synonyms, str):
+                self.aliases.add(synonyms)
+            else:
+                self.aliases.update(synonyms)
         setfield = setattr
         for i in _names: setfield(self, i, None)
         for i in _data: setfield(self, i, None)
         for i in _energy_handles: setfield(self, i, None)
         check_valid_ID(ID)
-        self._ID = ID
         self._phase_ref = phase_ref or phase
         self._CAS = CAS or ID
         for i,j in data.items(): setfield(self, '_' + i , j)
@@ -642,6 +662,11 @@ class Chemical:
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
         return self
 
+    @property
+    def charge(self):
+        """Charge of chemical as described in the chemcial formula."""
+        return charge_from_formula(self.formula)
+
     def copy(self, ID, CAS=None, **data):
         """
         Return a copy of the chemical with a new ID.
@@ -652,7 +677,7 @@ class Chemical:
         >>> Glucose = tmo.Chemical('Glucose')
         >>> Mannose = Glucose.copy('Mannose')
         >>> Mannose.show()
-        Chemical: Mannose (phase_ref='l')
+        Chemical: Mannose (phase_ref='s')
         [Names]  CAS: Mannose
                  InChI: None
                  InChI_key: None
@@ -666,18 +691,18 @@ class Chemical:
                  PSRK: <1CH2, 4CH, 5OH, 1CHO...
                  NIST: <Empty>
         [Data]   MW: 180.16 g/mol
-                 Tm: None
-                 Tb: 615.69 K
-                 Tt: None
+                 Tm: 419.15 K
+                 Tb: 419.15 K
+                 Tt: 419.15 K
                  Tc: 755 K
-                 Pt: None
+                 Pt: 0.21809 Pa
                  Pc: 4.82e+06 Pa
                  Vc: 0.000414 m^3/mol
                  Hf: -1.2711e+06 J/mol
                  S0: 0 J/K/mol
                  LHV: 2.5406e+06 J/mol
                  HHV: 2.8047e+06 J/mol
-                 Hfus: 0 J/mol
+                 Hfus: 19933 J/mol
                  Sfus: None
                  omega: 2.387
                  dipole: None
@@ -709,10 +734,10 @@ class Chemical:
         handles = (self._Psat, self._Hvap, self._sigma, self._epsilon,
                    self._V, self._Cn, self._mu, self._kappa)
         isa = isinstance
-        if self.CAS == self.ID:
-            label = self.CAS
-        else:
+        if add_ID and self.CAS != self.ID:
             label = f"{self.CAS} ({self.ID})"
+        else:
+            label = self.CAS
         for handle in handles:
             if isa(handle, PhaseHandle):
                 for i, j in handle:
@@ -856,12 +881,14 @@ class Chemical:
     ### Names ###
     
     @property
-    def synonyms(self):
-        """set[str] User-defined synonyms."""
-        return self._synonyms
-    @synonyms.setter
-    def synonyms(self, synonyms):
-        self._synonyms = set([synonyms]) if isinstance(synonyms, str) else set(synonyms)
+    def aliases(self):
+        """set[str] User-defined aliases."""
+        return self._aliases
+    @aliases.setter
+    def aliases(self, aliases):
+        self._aliases = set([aliases]) if isinstance(aliases, str) else set(aliases)
+    
+    synonyms = aliases
     
     @property
     def InChI(self):
@@ -1003,6 +1030,22 @@ class Chemical:
     @Hvap.setter
     def Hvap(self, value): 
         raise_helpful_handle_error('Hvap', self._Hvap)
+    
+    @property
+    def Hsub(self):
+        """Heat of sublimation [J/mol]."""
+        return self._Hsub
+    @Hsub.setter
+    def Hsub(self, value):
+        raise_helpful_handle_error('Hsub', self._Hsub)
+        
+    @property
+    def Psub(self):
+        """Sublimation pressure [Pa]."""
+        return self._Psub
+    @Psub.setter
+    def Psub(self, value):
+        raise_helpful_handle_error('Psub', self._Psub)
         
     @property
     def sigma(self): 
@@ -1318,8 +1361,8 @@ class Chemical:
               Tb=None, Tc=None, Pc=None, Vc=None, omega=None,
               Tt=None, Pt=None, Hf=None, S0=None, LHV=None, combustion=None,
               HHV=None, Hfus=None, dipole=None,
-              similarity_variable=None, iscyclic_aliphatic=None, synonyms=None,
-              *, metadata=None, phase=None):
+              similarity_variable=None, iscyclic_aliphatic=None, aliases=None,
+              synonyms=None, *, metadata=None, phase=None):
         """
         Reset all chemical properties.
 
@@ -1360,7 +1403,7 @@ class Chemical:
             MW = compute_molecular_weight(formula)
         self._init_names(CAS, smiles, InChI, InChI_key, 
                          pubchemid, iupac_name, common_name,
-                         formula, synonyms)
+                         formula, aliases, synonyms)
         self._init_groups(InChI_key)
         if CAS == '56-81-5': # TODO: Make this part of data
             self._Dortmund = DortmundGroupCounts({2: 2, 3: 1, 14: 2, 81: 1})
@@ -1405,7 +1448,7 @@ class Chemical:
     ### Initializers ###
     
     def _init_names(self, CAS, smiles, InChI, InChI_key,
-                    pubchemid, iupac_name, common_name, formula, synonyms):
+                    pubchemid, iupac_name, common_name, formula, aliases, synonyms):
         self._CAS = CAS
         self._smiles = smiles
         self._InChI = InChI
@@ -1413,7 +1456,12 @@ class Chemical:
         self._pubchemid = pubchemid
         self._iupac_name = iupac_name
         self._common_name = common_name
-        self.synonyms = synonyms or ()
+        self.aliases = aliases or ()
+        if synonyms: 
+            if isinstance(synonyms, str):
+                self.aliases.add(synonyms)
+            else:
+                self.aliases.update(synonyms)
         self._formula = formula
         
     def _init_groups(self, InChI_key):
@@ -1504,7 +1552,7 @@ class Chemical:
         self._V = V = PhaseTPHandle('V', Vs, Vl, Vg)
         if CAS in sugar_solid_densities: Vs.add_method(sugar_solid_densities[CAS])
         
-        # Heat capacity
+        # Sensible heats
         Cns = HeatCapacitySolid(CASRN=CAS, similarity_variable=similarity_variable, MW=MW)
         Cng = HeatCapacityGas(CASRN=CAS, MW=MW, similarity_variable=similarity_variable, 
                               iscyclic_aliphatic=iscyclic_aliphatic)
@@ -1512,11 +1560,19 @@ class Chemical:
                                  similarity_variable=similarity_variable, Cpgm=Cng)
         self._Cn = Cn = PhaseTHandle('Cn', Cns, Cnl, Cng)
         
-        # Heat of vaporization
+        # Latent heats
         Zl = CompressibilityFactor(Vl)
         Zg = CompressibilityFactor(Vg)
-        self._Hvap = EnthalpyVaporization(CASRN=CAS, Tb=Tb, Tc=Tc, Pc=Pc, omega=omega,
-                                          Zl=Zl, Zg=Zg, similarity_variable=similarity_variable, Psat=Psat)
+        self._Hvap = Hvap = EnthalpyVaporization(
+            CASRN=CAS, Tb=Tb, Tc=Tc, Pc=Pc, omega=omega,
+            Zl=Zl, Zg=Zg, similarity_variable=similarity_variable, Psat=Psat
+        )
+        self._Hsub = Hsub = EnthalpySublimation(
+            CASRN=CAS, Tm=Tm, Tt=Tt, Cpg=Cng, Cps=Cns, Hvap=Hvap
+        )
+        self._Psub = SublimationPressure(
+            CASRN=CAS, Tt=Tt, Pt=Pt, Hsub_t=Hsub(Tm) if Hsub and Tm else None,
+        )
         
         # Viscosity
         mul = ViscosityLiquid(CASRN=CAS, MW=MW, Tm=Tm, Tc=Tc, Pc=Pc, Vc=Vc, 
