@@ -19,6 +19,7 @@ from collections.abc import Iterable
 from .exceptions import DimensionError, InfeasibleRegion
 from chemicals.elements import array_to_atoms, symbol_to_index
 from . import utils
+from .constants import g
 
 __all__ = ('Stream', )
 
@@ -238,11 +239,14 @@ class Stream:
                    Ethanol  10
     
     """
-    __slots__ = ('_ID', '_imol', '_thermal_condition', '_thermo', '_streams',
-                 '_bubble_point_cache', '_dew_point_cache',
-                 '_vle_cache', '_lle_cache', '_sle_cache',
-                 '_sink', '_source', '_price', '_link', '_property_cache_key',
-                 '_property_cache', 'characterization_factors')
+    __slots__ = (
+        '_ID', '_imol', '_thermal_condition', '_thermo', '_streams',
+        '_bubble_point_cache', '_dew_point_cache',
+        '_vle_cache', '_lle_cache', '_sle_cache',
+        '_sink', '_source', '_price', '_islinked', '_property_cache_key',
+        '_property_cache', 'characterization_factors',
+        # '_velocity', '_height'
+    )
     line = 'Stream'
     
     #: [DisplayUnits] Units of measure for IPython display (class attribute)
@@ -257,13 +261,17 @@ class Stream:
 
     def __init__(self, ID= '', flow=(), phase='l', T=298.15, P=101325.,
                  units=None, price=0., total_flow=None, thermo=None, 
-                 characterization_factors=None, **chemical_flows):
+                 characterization_factors=None, 
+                 # velocity=0., height=0.,
+                 **chemical_flows):
         #: dict[obj, float] Characterization factors for life cycle assessment in impact / kg.
         self.characterization_factors = {} if characterization_factors is None else {}
         self._thermal_condition = tmo.ThermalCondition(T, P)
         thermo = self._load_thermo(thermo)
         chemicals = thermo.chemicals
         self.price = price
+        # self.velocity = velocity
+        # self.height = height
         if units:
             name, factor = self._get_flow_name_and_factor(units)
             if name == 'mass':
@@ -294,13 +302,13 @@ class Stream:
         self._sink = self._source = None # For BioSTEAM
         self.reset_cache()
         self._register(ID)
-        self._link = None
+        self._islinked = False
 
     def _reset_thermo(self, thermo):
         if thermo is self._thermo: return
         self._thermo = thermo
         self._imol.reset_chemicals(thermo.chemicals)
-        self._link = None
+        self._islinked = False
         self.reset_cache()
         if hasattr(self, '_streams'):
             for phase, stream in self._streams.items():
@@ -488,6 +496,38 @@ class Stream:
         else:
             raise AttributeError(f'price must be finite, not {price}')
     
+    # @property
+    # def velocity(self):
+    #     """[float] Velocity of stream [m/s]."""
+    #     return self._velocity
+    # @velocity.setter
+    # def velocity(self, velocity):
+    #     if np.isfinite(velocity):
+    #         self._velocity = float(velocity)
+    #     else:
+    #         raise AttributeError(f'velocity must be finite, not {velocity}')
+    
+    # @property
+    # def height(self):
+    #     """[float] Relative height of stream [m]."""
+    #     return self._height
+    # @height.setter
+    # def height(self, height):
+    #     if np.isfinite(height):
+    #         self._height = float(height)
+    #     else:
+    #         raise AttributeError(f'height must be finite, not {height}')
+    
+    # @property
+    # def potential_energy(self):
+    #     """[float] Potential energy flow rate [kW]"""
+    #     return (g * self.height * self.F_mass) / 3.6e6
+    
+    # @property
+    # def kinetic_energy(self):
+    #     """[float] Kinetic energy flow rate [kW]"""
+    #     return 0.5 * self.F_mass / 3.6e6 * self._velocity * self._velocity
+    
     def isempty(self):
         """
         Return whether or not stream is empty.
@@ -591,7 +631,7 @@ class Stream:
         """Reset cache regarding equilibrium methods."""
         self._bubble_point_cache = eq.BubblePointCache()
         self._dew_point_cache = eq.DewPointCache()
-        self._property_cache_key = None, None
+        self._property_cache_key = None, None, None
         self._property_cache = {}
 
     @classmethod
@@ -869,7 +909,7 @@ class Stream:
     @property
     def H(self):
         """[float] Enthalpy flow rate in kJ/hr."""
-        H = self._get_property_cache('H')
+        H = self._get_property_cache('H', True)
         if H is None:
             self._property_cache['H'] = H = self.mixture.H(
                 self.phase, self.mol, *self._thermal_condition
@@ -896,7 +936,12 @@ class Stream:
     @property
     def S(self):
         """[float] Absolute entropy flow rate in kJ/hr."""
-        return self.mixture.S(self.phase, self.mol, *self._thermal_condition)
+        S = self._get_property_cache('S', True)
+        if S is None:
+            self._property_cache['S'] = S = self.mixture.S(
+                self.phase, self.mol, *self._thermal_condition
+            )
+        return S
     
     @property
     def Hnet(self):
@@ -920,7 +965,7 @@ class Stream:
         """[float] Enthalpy of vaporization flow rate in kJ/hr."""
         mol = self.mol
         T = self._thermal_condition._T
-        Hvap = self._get_property_cache('Hvap')
+        Hvap = self._get_property_cache('Hvap', True)
         if Hvap is None:
             self._property_cache['Hvap'] = Hvap = sum([
                 i*j.Hvap(T) for i,j in zip(mol, self.chemicals)
@@ -928,24 +973,32 @@ class Stream:
             ])
         return Hvap
     
-    def _get_property_cache(self, name):
+    def _get_property_cache(self, name, flow=False):
         property_cache = self._property_cache
         thermal_condition = self._thermal_condition
         imol = self._imol
         data = imol._data
+        total = data.sum()
+        if total == 0.: return 0.
+        composition = data / total
         literal = (imol._phase._phase, thermal_condition._T, thermal_condition._P)
-        last_literal, last_data = self._property_cache_key
-        if literal == last_literal and (data == last_data).all():
-            return property_cache.get(name) 
+        last_literal, last_composition, last_total = self._property_cache_key
+        if literal == last_literal and (composition == last_composition).all():
+            prop = property_cache.get(name)
+            if not prop: return prop
+            if flow:
+                return prop * total / last_total
+            else:
+                return prop
         else:
-            self._property_cache_key = (literal, data.copy())
+            self._property_cache_key = (literal, composition, total)
             property_cache.clear()
             return None
     
     @property
     def C(self):
         """[float] Heat capacity flow rate in kJ/hr."""
-        C = self._get_property_cache('C')
+        C = self._get_property_cache('C', True)
         if C is None:
             self._property_cache['C'] = C = self.mixture.Cn(self.phase, self.mol, self.T)
         return C
@@ -1305,6 +1358,11 @@ class Stream:
         TP : bool, defaults to True
             Whether to link the temperature and pressure.
         
+        See Also
+        --------
+        :obj:`~Stream.flow_proxy`
+        :obj:`~Stream.proxy`
+        
         Examples
         --------
         >>> import thermosteam as tmo
@@ -1325,23 +1383,20 @@ class Stream:
             at_unit = f" at unit {self.source}" if self.source is other.sink else ""
             raise RuntimeError(f"stream {self} cannot link with stream {other}" + at_unit
                                + "; streams must have the same class to link")
-        if self._link and not (self.source is other.sink or self.sink is other.source):
+        if self._islinked and not (self.source is other.sink or self.sink is other.source):
             raise RuntimeError(f"stream {self} cannot link with stream {other};"
-                               f" {self} already linked with {self._link}")
+                               f" {self} already linked")
         if TP and flow and (phase or self._imol._data.ndim == 2):
             self._imol._data_cache = other._imol._data_cache
         else:
             self._imol._data_cache.clear()
-        
         if TP:
             self._thermal_condition = other._thermal_condition
         if flow:
             self._imol._data = other._imol._data
         if phase and self._imol._data.ndim == 1:
             self._imol._phase = other._imol._phase
-        
-        self._link = other
-        other._link = self
+        self._islinked = other._islinked = True
             
     def unlink(self):
         """
@@ -1369,13 +1424,13 @@ class Stream:
         imol = self._imol
         if hasattr(imol, '_phase') and isinstance(imol._phase, tmo._phase.LockedPhase):
             raise RuntimeError('phase is locked; stream cannot be unlinked')
-        if self._link:
+        if self._islinked:
             imol._data_cache.clear()
             imol._data = imol._data.copy()
             imol._phase = imol._phase.copy()
             self._thermal_condition = self._thermal_condition.copy()
             self.reset_cache()
-            self._link = None
+            self._islinked = False
         
     
     def copy_like(self, other):
@@ -1595,7 +1650,8 @@ class Stream:
         """
         cls = self.__class__
         new = cls.__new__(cls)
-        new._link = new._sink = new._source = None
+        new._islinked = False
+        new._sink = new._source = None
         new.characterization_factors = {}
         new._thermo = thermo or self._thermo
         new._imol = self._imol.copy()
@@ -1612,6 +1668,11 @@ class Stream:
         """
         Return a new stream that shares flow rate data with this one.
         
+        See Also
+        --------
+        :obj:`~Stream.link_with`
+        :obj:`~Stream.proxy`
+        
         Examples
         --------
         >>> import thermosteam as tmo
@@ -1632,12 +1693,21 @@ class Stream:
         new._thermal_condition = self._thermal_condition.copy()
         new.reset_cache()
         new.characterization_factors = {}
-        new._link = self
+        self._islinked = new._islinked = True
         return new
     
     def proxy(self, ID=None):
         """
-        Return a new stream that shares all data with this one.
+        Return a new stream that shares all thermochemical data with this one.
+        
+        See Also
+        --------
+        :obj:`~Stream.link_with`
+        :obj:`~Stream.flow_proxy`
+        
+        Warning
+        -------
+        Price and characterization factor data is not shared
         
         Examples
         --------
@@ -1663,7 +1733,8 @@ class Stream:
         new._dew_point_cache = self._dew_point_cache
         try: new._vle_cache = self._vle_cache
         except AttributeError: pass
-        new._link = self
+        new.characterization_factors = {}
+        self._islinked = new._islinked = True
         return new
     
     def empty(self):
@@ -2100,17 +2171,15 @@ class Stream:
         other.copy_like(liquid)
         self.T = other.T = ms.T
         
-        
-        
     ### Casting ###
     
     @property
-    def link(self):
+    def islinked(self):
         """
-        [Stream] Data on the thermal condition and material flow rates may 
-        be shared with this stream.
+        [bool] Whether data regarding the thermal condition, material flow rates,
+        and phases are shared with other streams.
         """
-        return self._link
+        return self._islinked
     
     @property
     def phases(self):
@@ -2119,7 +2188,7 @@ class Stream:
     @phases.setter
     def phases(self, phases):
         if self.phases == phases: return
-        if self._link: self.unlink()
+        if self._islinked: self.unlink()
         if len(phases) == 1:
             self.phase = phases[0]
         else:
