@@ -423,7 +423,9 @@ class ChemicalIndexer(Indexer):
     def get_phase_and_composition(self):
         """Return phase and composition."""
         data = self._data
-        return self.phase, data / data.sum()
+        total = data.sum()
+        if total <= 0.: raise RuntimeError(f"'{tmo.settings.phase_names[self.phase]}' phase does not exist")
+        return self.phase, data / total
     
     def __format__(self, tabs=""):
         if not tabs: tabs = 1
@@ -964,7 +966,8 @@ ChemicalVolumetricFlowIndexer, VolumetricFlowIndexer = _new_Indexer('VolumetricF
 
 # %% Mass flow properties
 
-@PropertyFactory(slots=('name', 'mol', 'index', 'MW'))
+@PropertyFactory(slots=('name', 'mol', 'index', 'MW'),
+                 units='kg/hr')
 def MassFlowProperty(self):
     """Mass flow (kg/hr)."""
     return self.mol[self.index] * self.MW
@@ -1014,7 +1017,8 @@ MolarFlowIndexer.by_mass = by_mass; del by_mass
 # %% Volumetric flow properties
 
 @PropertyFactory(slots=('name', 'mol', 'index', 'V',
-                        'TP', 'phase', 'phase_container'))
+                        'TP', 'phase', 'phase_container'),
+                 units='m^3/hr')
 def VolumetricFlowProperty(self):
     """Volumetric flow (m^3/hr)."""
     f_mol = self.mol[self.index] 
@@ -1040,7 +1044,7 @@ def by_volume(self, TP):
     
     """
     try:
-        vol = self._data_cache[TP]
+        vol = self._data_cache['vol', TP]
     except:
         chemicals = self.chemicals
         mol = self.data
@@ -1048,7 +1052,7 @@ def by_volume(self, TP):
         for i, chem in enumerate(chemicals):
             vol[i] = VolumetricFlowProperty(chem.ID, mol, i, chem.V,
                                             TP, None, self._phase)
-        self._data_cache[TP] = \
+        self._data_cache['vol', TP] = \
         vol = ChemicalVolumetricFlowIndexer.from_data(property_array(vol),
                                                       self._phase, chemicals,
                                                       False)
